package de.hamster.scratch;

import java.util.ArrayList;
import java.util.HashMap;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import de.hamster.scratch.elements.BooleanMethodObject;
import de.hamster.scratch.elements.VoidObject;
import de.hamster.scratch.gui.DeleteMethodException;
import de.hamster.scratch.gui.InvalidIdentifierException;
import de.hamster.scratch.gui.RenameMethodException;

/**
 * Der StorageController beinhaltet alle Inhalte verschiedener
 * Methoden. Die Inhalte knnen anhand des Methodennamen (String)
 * aus einer HashMap herausgefunden werden. Dieser ist auch fr
 * das Speichern und Laden der Methoden aus XML Dateien verantworlich.
 * @author HackZ
 *
 */
public class StorageController {
	public static ArrayList<String> constantNames;
	public static ArrayList<String> constantReserved;
	
	private HashMap<String, Method> voidMethods;
	private ArrayList<Method> voidMethodsList;
	private HashMap<String, Method> booleanMethods;
	private ArrayList<Method> booleanMethodsList;
	
	/**
	 * Erstellt einen neuen StorageController, in dem die Methode
	 * main gleich erstellt wird und als offen deklariert wird.
	 */
	public StorageController() {
		fillConstantNames();
		voidMethods = new HashMap<String, Method>();
		voidMethodsList = new ArrayList<Method>();
		booleanMethods = new HashMap<String, Method>();
		booleanMethodsList = new ArrayList<Method>();
		
		addVoidMethod("main");
		getVoidMethod("main").setOpened(true);
	}
	
	/**
	 * Das statische Attribut constatnNames wird mit verschiedenen
	 * Konstanten gefllt. Dies geschieht nur einmalig beim ersten
	 * Aufruf, sollte ein weiteres ScratchPanel geffnet werden, so
	 * sind die Varieblen bereits enthalten und die Funktion wird
	 * abgebrochen.
	 */
	private void fillConstantNames() {
		if (constantNames != null)
			return;
		
		constantNames = new ArrayList<String>();
		constantReserved = new ArrayList<String>();
		
		constantNames.add("vor");
		constantNames.add("linksUm");
		constantNames.add("gib");
		constantNames.add("nimm");
		constantNames.add("vornFrei");
		constantNames.add("kornDa");
		constantNames.add("maulLeer");
		
		constantReserved.add("wahr");
		constantReserved.add("falsch");
		constantReserved.add("true");
		constantReserved.add("false");
		constantReserved.add("falls");
		constantReserved.add("solange");
		constantReserved.add("tueSolange");
		constantReserved.add("null");
		constantReserved.add("und");
		constantReserved.add("oder");
		constantReserved.add("nicht");
		
		constantReserved.add("abstract");
		constantReserved.add("assert");
		constantReserved.add("boolean");
		constantReserved.add("break");
		constantReserved.add("byte");
		constantReserved.add("case");
		constantReserved.add("catch");
		constantReserved.add("char");
		constantReserved.add("class");
		constantReserved.add("const");
		constantReserved.add("continue");
		constantReserved.add("default");
		constantReserved.add("do");
		constantReserved.add("double");
		constantReserved.add("else");
		constantReserved.add("enum");
		constantReserved.add("extends");
		constantReserved.add("final");
		constantReserved.add("finally");
		constantReserved.add("float");
		constantReserved.add("for");
		constantReserved.add("goto");
		constantReserved.add("if");
		constantReserved.add("implements");
		constantReserved.add("import");
		constantReserved.add("instanceof");
		constantReserved.add("int");
		constantReserved.add("interface");
		constantReserved.add("long");
		constantReserved.add("native");
		constantReserved.add("new");
		constantReserved.add("package");
		constantReserved.add("private");
		constantReserved.add("protected");
		constantReserved.add("public");
		constantReserved.add("return");
		constantReserved.add("short");
		constantReserved.add("static");
		constantReserved.add("strictfp");
		constantReserved.add("super");
		constantReserved.add("switch");
		constantReserved.add("synchronized");
		constantReserved.add("this");
		constantReserved.add("throw");
		constantReserved.add("throws");
		constantReserved.add("transient");
		constantReserved.add("try");
		constantReserved.add("void");
		constantReserved.add("volatile");
		constantReserved.add("while");
	}
	
	/**
	 * Liefert alle Methoden als ArrayList zurck, die geffnet sind. Ist
	 * sinnvoll beim Laden des Programms, um festzustellen welche Tabs offen
	 * sind und diese im Programm zu ffnen.
	 * @return
	 */
	public ArrayList<String> getOpenedMethods() {
		ArrayList<String> res = new ArrayList<String>();
		
		for (Method m : voidMethodsList)
			if (m.isOpened())
				res.add(m.getName());
		
		for (Method m : booleanMethodsList)
			if (m.isOpened())
				res.add(m.getName());
		
		return res;
	}

	/**
	 * Setzt fest, ob die Methode mit dem bergebenen Namen offen oder
	 * geschlossen ist.
	 * @param name
	 * Name der Methode.
	 * @param opened
	 */
	public void setOpened(String name, boolean opened) {
		Method m = getMethod(name);
		
		if (m == null)
			return;
		
		m.setOpened(opened);
	}
	
	/**
	 * Fgt eine neue void-Methode hinzu. Der name darf noch nicht
	 * vorhanden sein.
	 * @param name
	 */
	public void addVoidMethod(String name) {
		if (voidMethods.containsKey(name))
			return;
		
		Method m = new Method(name, Renderable.Type.VOID);
		voidMethods.put(name, m);
		voidMethodsList.add(m);
	}
	
	/**
	 * Liefert die Methode mit dem bergebenen Namen.
	 * @param name
	 * Name der Methode
	 * @return
	 */
	public Method getVoidMethod(String name) {
		if (!voidMethods.containsKey(name))
			return null;
		
		return voidMethods.get(name);
	}
	
	/**
	 * Liefert alle void-Methoden als ArrayListe. Wird verwendet,
	 * um alle void-Methoden in der ElementListe anzuzeigen.
	 * @return
	 */
	public ArrayList<Renderable> getAllVoidMethods() {
		ArrayList<Renderable> result = new ArrayList<Renderable>();
		for (Method m : voidMethodsList)
			result.add(new VoidObject(m.getName(), new ArrayList<Renderable.Type>()));
		
		return result;
	}
	
	/**
	 * Fgt eine neue boolean-Methode hinzu. Der name darf noch nicht
	 * vorhanden sein.
	 * @param name
	 */
	public void addBooleanMethod(String name) {
		if (booleanMethods.containsKey(name))
			return;
		
		Method m = new Method(name, Renderable.Type.BOOLEAN);
		booleanMethods.put(name, m);
		booleanMethodsList.add(m);
	}
	
	/**
	 * Liefert die Methode mit dem bergebenen Namen
	 * @param name
	 * Name der Methode
	 * @return
	 */
	public Method getBooleanMethod(String name) {
		if (!booleanMethods.containsKey(name))
			return null;
		
		return booleanMethods.get(name);
	}
	
	/**
	 * Liefert alle boolean-Methoden als ArrayListe. Wird verwendet,
	 * um alle boolean-Methoden in der ElementListe anzuzeigen.
	 * @return
	 */
	public ArrayList<Renderable> getAllBooleanMethods() {
		ArrayList<Renderable> result = new ArrayList<Renderable>();
		for (Method m : booleanMethodsList)
			result.add(new BooleanMethodObject(m.getName(), new ArrayList<Renderable.Type>()));
		
		return result;
	}
	
	/**
	 * Liefer die void- oder boolean-Methode mit dem bergebenen Namen.
	 * @param name
	 * @return
	 */
	public Method getMethod(String name) {
		Method m = getVoidMethod(name);
		if (m != null)
			return m;
		
		return getBooleanMethod(name);
	}

	/**
	 * Liefert das erste ausfhrbare Renderable aus der main-Methode,
	 * mit dem das Programm anfngt.
	 * @return
	 */
	public Renderable getMainRoot() {
		Method main = voidMethods.get("main");
		return main.getRootElement();
	}

	/**
	 * berprft, ob der bergebene Name noch nicht vergeben ist
	 * oder wirft eine Exception mit der passenden Fehlerbeschreibung.
	 * Es wird dabei der name jedoch noch nicht registriert, sodern
	 * dient nur der reinen berprfung.
	 * @param name
	 * name eine Methode, die erstellt werden soll
	 * @throws InvalidIdentifierException
	 * wird geworfen, falls
	 * <ol>
	 * <li>Der Bezeichner hat keine gltige Javakonvention.</li>
	 * <li>Der Bezeichner ist bereits vergeben.</li>
	 * <li>Der Bezeichner ist ein reserviertes Javakeyword.</li>
	 * </ol>
	 */
	public void allocName(String name) throws InvalidIdentifierException {
		ScratchUtils.checkJavaIdentifier(name);
		
		if (getBooleanMethod(name) != null)
			throw new InvalidIdentifierException("Dieser Bezeichnername ist bereits vergeben!");
		
		if (getVoidMethod(name) != null)
			throw new InvalidIdentifierException("Dieser Bezeichnername ist bereits vergeben!");
		
		for (String s : constantNames)
			if (s.equals(name))
				throw new InvalidIdentifierException("Dieser Bezeichnername ist bereits vergeben!");
		
		for (String s : constantReserved)
			if (s.equals(name))
				throw new InvalidIdentifierException("Keywords sind reserviert und drfen nicht als Bezeichner verwendet werden!"); 
	}
	
	/**
	 * berprft, ob die Methode mit dem bergebenen Namen
	 * im Programm verwendet wird.
	 * @param name
	 * Name der Methode, die getestet werden soll.
	 * @return
	 * true, wenn diese Methode im Programm verwendet wird.
	 */
	public boolean inUse(String name) {
		// Alle Void Methoden testen
		for (Method m : voidMethodsList)
			if (!m.getName().equals(name) && m.inUse(name))
				return true;
		
		for (Method m : booleanMethodsList)
			if (!m.getName().equals(name) && m.inUse(name))
				return true;
		
		return false;
	}
	
	/**
	 * Lscht die Methode mit dem bergebenen Namen
	 * @param name
	 * @throws DeleteMethodException
	 * wird geworfen, falls versucht wird
	 * <ol>
	 * <li>die Methode <tt>main</tt> zu lschen.</li>
	 * <li>eine Methode zu lschen, die in einer Anderen Methode noch verwendet wird.</li>
	 * <li>eine feste Methode oder einen Javakey zu lschen.</li>
	 * </ol>
	 */
	public void deleteMethod(String name) throws DeleteMethodException {
		if (name.equals("main"))
			throw new DeleteMethodException("Die Methode \"main\" ist der Startpunkt des Programms und kann weder gelscht, noch umbenannt werden!");
		
		if (inUse(name))
			throw new DeleteMethodException("Die Methode " + name + " wird bereits im Programm verwendet. Es knnen nur nicht verwendete Methoden gelscht werden!");
		
		// Versuche void Methode zu lschen
		if (voidMethods.containsKey(name)) {
			Method temp = voidMethods.get(name);
			voidMethods.remove(name);
			voidMethodsList.remove(temp);
			return;
		}
		
		// Versuche boolean Methode zu lschen
		if (booleanMethods.containsKey(name)) {
			Method temp = booleanMethods.get(name);
			booleanMethods.remove(name);
			booleanMethodsList.remove(temp);
			return;
		}
		
		// Name ist wahrscheinlich ein Key oder feste Methode (vor, linksUm...)
		throw new DeleteMethodException("Methoden aus dem Standardwortschatz des Hamsters, sowie Javakeywrter drfen nicht gelscht werden!");
	}
	
	/**
	 * Benennt eine Methode um
	 * @param fromName
	 * @param toName
	 * @throws RenameMethodException
	 * Wird geworfen falls versucht wird
	 * <ol>
	 * <li>die Methode <tt>main</tt> umzubenennen.</li>
	 * <li>der neue Methodenname keine gltige Javakonvetion hat.</li>
	 * <li>der neue Methodenname bereits vergeben ist.</li>
	 * <li>eine feste Methode oder einen Javakey umzubenennen.</li>
	 * </ol>
	 */
	public void rename(String fromName, String toName) throws RenameMethodException {
		if (fromName.equals("main"))
			throw new RenameMethodException("Die Methode \"main\" ist der Startpunkt des Programms und kann weder gelscht, noch umbenannt werden!");
		
		try {
			this.allocName(toName);
		} catch (InvalidIdentifierException e) {
			throw new RenameMethodException(e.getMessage());
		}
		
		for (Method m : voidMethodsList)
			m.rename(fromName, toName);
		
		for (Method m : booleanMethodsList)
			m.rename(fromName, toName);
		
		if (voidMethods.containsKey(fromName)) {
			Method temp = voidMethods.remove(fromName);
			voidMethods.put(toName, temp);
		}
		
		if (booleanMethods.containsKey(fromName)) {
			Method temp = booleanMethods.remove(fromName);
			booleanMethods.put(toName, temp);
		}
	}

	/**
	 * berprft, ob die bergebene Methode existiert
	 * @param name
	 * @return
	 * true, wenn eine selbst erstellte Methode existiert.
	 * False, wenn die Methode nicht existiert, oder ein
	 * Javakeyword ist oder zum Sprachwortschatz des Hamsters gehrt
	 */
	public boolean existsMethod(String name) {
		for (Method m : voidMethodsList)
			if (m.getName().equals(name))
				return true;
		
		for (Method m : booleanMethodsList)
			if (m.getName().equals(name))
				return true;
		
		return false;
	}

	/**
	 * Schreibt den Inhalt des StorageControllers in ein
	 * XML-Document. Das XML-Document muss von ausserhalb
	 * bereits vorbereitet worden sein, mit dem Startelement.
	 * @param writer
	 * Der writer, der bereits das Startelement enthlt.
	 * @throws XMLStreamException 
	 */
	public void toXML(XMLStreamWriter writer) throws XMLStreamException {
		for (Method m : voidMethodsList)
			m.toXML(writer);
		
		for (Method m : booleanMethodsList)
			m.toXML(writer);
	}

	/**
	 * Ldt das Programm anhand der bergebenen NodeList. Dies sind
	 * alle Kinderelemente des Wurzelelements <tt>SCRATCHPROGRAM</tt>.
	 * @param methods
	 * Kinderelemente von <tt>SCRATCHPROGRAM</tt>
	 */
	public void loadProgram(NodeList methods) {
		voidMethods.clear();
		voidMethodsList.clear();
		booleanMethods.clear();
		booleanMethodsList.clear();
		
		for (int i = 0; i < methods.getLength(); i++) {
			Element method = (Element)methods.item(i);
			String name = method.getAttribute("NAME");
			String type = method.getAttribute("TYPE");
			String opened = method.getAttribute("OPENED");
			
			Method m = null;
			if (type.equals("VOID")) {
				addVoidMethod(name);
				m = getVoidMethod(name);
			}
			else {
				addBooleanMethod(name);
				m = getBooleanMethod(name);
			}
			
			m.loadProgram(method.getChildNodes());
			m.setOpened(opened.equals("T"));
		}
	}

	/**
	 * bernimmt den Inhalt des bergebenen Controllers in diesem
	 * @param program
	 */
	public void setNewController(StorageController controller) {
		this.booleanMethods = controller.booleanMethods;
		this.booleanMethodsList = controller.booleanMethodsList;
		this.voidMethods = controller.voidMethods;
		this.voidMethodsList = controller.voidMethodsList;
	}
}
