package de.hamster.scratch.elements.booleans;

import java.awt.Graphics;

import de.hamster.scratch.BoundingBox;
import de.hamster.scratch.DockingBox;
import de.hamster.scratch.Renderable;
import de.hamster.scratch.ScratchProgram;
import de.hamster.scratch.ScratchUtils;
import de.hamster.scratch.elements.BooleanObject;
import de.hamster.scratch.elements.voids.FunctionResultException;

public class AndBooleanObject extends BooleanObject {
	public static int START_WIDTH = 8;
	public static int TEXT_SPACING = 6;
	public static int MIN_WIDTH = 25;
	public static int MIN_HEIGHT = 13;
	public static String TEXT = "und";
	
	private int textWidth;
	
	public AndBooleanObject() {
		super(false);
		
		textWidth = ScratchUtils.getTextWidth(TEXT, TEXT_FONT);
		
		maxChilds = 2;
		DockingBox leftBool = new DockingBox(START_WIDTH, 2, MIN_WIDTH, MIN_HEIGHT, START_WIDTH, 2, Type.BOOLEAN);
		dockings.put(0, leftBool);
		DockingBox rightBool = new DockingBox(0, 0, 0, 0, 0, 0, Type.BOOLEAN);
		dockings.put(1, rightBool);
		
		updateBounds();
	}
	
	@Override
	public String getName() {
		return "und";
	}
	
	@Override
	public void render(Graphics g) {
		super.render(g);
		
		g.setColor(COLOR_TEXT);
		g.setFont(TEXT_FONT);
		int textHeight = 7;
	    g.drawString(TEXT, bounding.getX() + START_WIDTH + dockings.get(0).getWidth() + TEXT_SPACING, bounding.getY() + textHeight + (bounding.getHeight() - textHeight) / 2);
	}
	
	@Override
	public void updateBounds() {
		int w = 2 * (START_WIDTH + TEXT_SPACING) + textWidth;
		int h = MIN_HEIGHT;
		
		// Update breiten und Hhen der DockingBoxen
		for (int i = 0; i < maxChilds; i++) {
			if (childs.get(i) != null) {
				BoundingBox temp = childs.get(i).getGlobalBounding();
				w += temp.getWidth();
				h = (h < temp.getHeight() + 4) ? temp.getHeight() + 4 : h;
				dockings.get(i).setSize(temp.getWidth(), temp.getHeight());
			}
			else {
				w += dockings.get(i).getType().getWidth();
				h = (h < dockings.get(i).getType().getHeight() + 4) ? dockings.get(i).getType().getHeight() + 4 : h;
				dockings.get(i).setSize(dockings.get(i).getType().getWidth(), dockings.get(i).getType().getHeight());
			}
		}
		
		// Position der rechten DockBox
		dockings.get(1).setPosition(START_WIDTH + TEXT_SPACING * 2 + textWidth + dockings.get(0).getWidth(), 0);
		dockings.get(1).setDockX(START_WIDTH + TEXT_SPACING * 2 + textWidth + dockings.get(0).getWidth());
		
		// Verticale Ausrichtung der Dockings anpassen
		for (int i = 0; i < maxChilds; i++) {
			dockings.get(i).setPosition(dockings.get(i).getX(), (h - dockings.get(i).getHeight()) / 2);
			dockings.get(i).setDockY((h - dockings.get(i).getHeight()) / 2);
		}
		
		bounding.setSize(w, h);
		
		if (parent != null)
			parent.updateBounds();
	}

	@Override
	public Renderable clone() {
		AndBooleanObject temp = new AndBooleanObject();
		return temp;
	}

	@Override
	public Object performImplementation(ScratchProgram program) throws FunctionResultException {
		// Linkes Boolean auswerten
		boolean leftBool = true;
		if (childs.get(0) != null)
			leftBool = (Boolean)childs.get(0).perform(program);
		
		// Rechtes Boolean auswerten
		boolean rightBool = true;
		if (childs.get(1) != null)
			rightBool = (Boolean)childs.get(1).perform(program);
		
		// Gesamte Auswertung
		return (leftBool && rightBool);
	}
}
